package oracle.forms.fd;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;

import java.io.File;

import java.lang.reflect.Method;

import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.border.Border;

import oracle.ewt.event.AnyEventListener;

import oracle.forms.engine.Main;
import oracle.forms.handler.IHandler;
import oracle.forms.properties.ID;
import oracle.forms.ui.VBean;

import org.jdic.web.BrComponent;


/**
 * A Forms JavaBean to display a Google Map.
 *
 * February 2015
 * @author Francois Degrelle
 * @version 1.0
 */

public class GMapBrowser extends VBean {
    //
    // Variables declaration
    //
    protected static final ID pSetUrl         = ID.registerProperty("SET_URL");
    protected static final ID pSetHTMLContent = ID.registerProperty("SET_HTML_CONTENT");
    protected static final ID pSetJavascript  = ID.registerProperty("SET_JAVASCRIPT");
    protected static final ID pSetMAPFnc      = ID.registerProperty("SET_MAP_FNC");
    protected static final ID pSetLog         = ID.registerProperty("SET_LOG");    
    protected static final ID pGetValue       = ID.registerProperty("GET_VALUE");
    protected static final ID pSetBorder      = ID.registerProperty("SET_BORDER");
    protected static final ID pSetNavigation  = ID.registerProperty("SET_NAVIGATION");
    protected static final ID pInfoBean       = ID.registerProperty("INFOBEAN");

    private oracle.forms.ui.ExtendedFrame ef;
    private oracle.ewt.swing.JBufferedFrame winMDI;
    private Main formsMain = null;
    private Frame formsTopFrame = null;
    private int x, y, w, h;
    private org.jdic.web.BrComponent webBrowser;
    private JPanel panel;
    private boolean bLog = false;
    private String sMapValue;


    /** Creates new Embedded HTML Browser */
    public GMapBrowser() {
        super();
        BrComponent.DESIGN_MODE = false;
        BrComponent.setDefaultPaintAlgorithm(BrComponent.PAINT_NATIVE);
        webBrowser = new org.jdic.web.BrComponent();
        Rectangle rec = this.getBounds();
        x = (int)rec.getX();
        y = (int)rec.getY();
        w = (int)rec.getWidth();
        h = (int)rec.getHeight();
        panel = new JPanel();
        panel.setLayout(new BorderLayout());
        panel.setPreferredSize(this.getPreferredSize());
        panel.add(webBrowser, BorderLayout.CENTER);
        webBrowser.setVisible(false);
        add(panel);
        getRect();
    }

    public void init(IHandler handler) {
        super.init(handler);
        // get the Forms Main class
        try {
            Method method = handler.getClass().getMethod("getApplet", new Class[0]);
            Object applet = method.invoke(handler, new Object[0]);
            if (applet instanceof Main) {
                formsMain = (Main)applet;
            }
        } catch (Exception ex) {
            ;
        }
        formsTopFrame = formsMain.getFrame();
        webBrowser.setBounds(x, y, w, h);
    }


    /******************************
     *  Set the bean properties   *
     *****************************/
    public boolean setProperty(ID pId, Object pValue) {
        // set the url
        if (pId == pSetUrl) 
        {
            URL url = null;
            String s = (String)pValue;
            webBrowser.setVisible(true);
            webBrowser.setBounds(x, y, w, h);
            try {
                if (!s.startsWith("http")) {
                    File f = new File(s);
                    url = new URL("file:///" + f.getAbsolutePath());
                    webBrowser.setURL(url);
                } else if (s.startsWith("http")) {
                    url = new URL(s);
                    webBrowser.setURL(url);
                } else
                    log("protocole not recognized : " + s);
                log(url.toString());
                webBrowser.setURL(url);

            } catch (Exception e) {
                System.out.println(e.getMessage());
            }
            return true;
        }
        // set the HTML content
        else if (pId == pSetHTMLContent) 
        {
            String s = (String)pValue;
            log("Set HTML=" + s);
            webBrowser.setVisible(true);
            try {
                webBrowser.setHTML(s);
                webBrowser.repaint();
            } catch (Exception ex) {
                System.out.println(ex.getMessage());
            }
            return true;
        }
        // set Javascript expression
        else if (pId == pSetJavascript) 
        {
            String s = (String)pValue;
            log("Set JS=" + s);
            try {
                webBrowser.execJS(s);
            } catch (Exception ex) {
                System.out.println(ex.getMessage());
            } finally{
                ;
            }
            return true;
        }
        // set map function
        else if (pId == pSetMAPFnc) 
        {
            String s = (String)pValue;
            log("Set MAP Fnc=" + s);
            try {
                sMapValue = webBrowser.execJS(s);
                log("value:" + sMapValue);
            } catch (Exception ex) {
                System.out.println(ex.getMessage());
            }
            return true;
        }
        // set the panel border
        else if (pId == pSetBorder) 
        {
            String s = (String)pValue;
            log("Set Border=" + s);
            if (s.equalsIgnoreCase("false")) {
                Border emptyBorder = BorderFactory.createEmptyBorder(0, 0, 0, 0);
                panel.setBorder(emptyBorder);
            } else if (s.equalsIgnoreCase("true")) {
                Border border = BorderFactory.createEtchedBorder();
                panel.setBorder(border);
            }
            return true;
        }
        // turn log on/off        
        else if (pId == pSetLog) 
        {
            String s = (String)pValue;
            log("Set Log=" + s);
            if (s.equalsIgnoreCase("false")) {
                bLog= false;
            } else if (s.equalsIgnoreCase("true")) {
                bLog= true;
            }
            return true;
        }        
        
        /***********************
        *  Navigation action  *
        **********************/
        else if (pId == pSetNavigation)
        {
            String s = (String)pValue;
            log("Navigation=" + s);
            if (s.equalsIgnoreCase("back"))
                webBrowser.back();
            else if (s.equalsIgnoreCase("forward"))
                webBrowser.forward();
            else if (s.equalsIgnoreCase("refresh"))
                webBrowser.refresh();
            return true;
        }
        /********************************************
        *   get the window that handle the bean    *
        *******************************************/
        else if (pId == pInfoBean) {
            boolean bCont = true;
            Container cont = this.getParent();
            String s = "";
            while (cont != null) {
                // search for the window
                s = "" + cont.getClass();
                if (s.indexOf("oracle.forms.ui.ExtendedFrame") > -1) {
                    /*********************
                    *   current window  *
                    ********************/
                    ef = (oracle.forms.ui.ExtendedFrame)cont;
                    log("------> ExtendedFrame title : " + ef.getTitle());
                    // add the listener to the window
                    ef.addAnyEventListener(ael);
                }
                cont = cont.getParent();
            }
            return true;
        } else {
            return super.setProperty(pId, pValue);
        }
    }

    /******************************
     *  Get the bean properties   *
     *****************************/    
    public Object getProperty(ID property) {
        if (property == pGetValue) {
            log("getProperty() value:" + "" + sMapValue);
            return "" + sMapValue;
        }
        return super.getProperty(property);
    }

    /** get the bean panel bounds */
    void getRect() {
        Rectangle rec = this.getBounds();
        x = (int)rec.getX();
        y = (int)rec.getY();
        w = (int)rec.getWidth();
        h = (int)rec.getHeight();
        log("-->" + rec);
    }

    AnyEventListener ael = new AnyEventListener() {
        public void actionPerformed(ActionEvent actionEvent) {
            String keyText = actionEvent.getActionCommand();
        }

        public void processEventStart(java.util.EventObject eo) {
            String keyText = eo.toString();
            if (keyText.indexOf("COMPONENT_MOVED") > -1) {
                getRect();
                webBrowser.setLocation(x, y);
                webBrowser.repaint();
            }
        }

        public void processEventEnd(java.util.EventObject eo) {
            String keyText = eo.toString();
        }
    };

    void log(String msg) {
        if (bLog) {
            System.out.println("GMapBrowser: " + msg);
        }
    }

}
